@@ -2,7 +2,7 @@ package Brackup::Target::Amazon;
 use strict;
 use warnings;
 use base 'Brackup::Target';
-use Net::Amazon::S3;
+use Net::Amazon::S3 0.31;
 use DBD::SQLite;
 
 sub new {
@@ -16,7 +16,7 @@ sub new {
 	eval {
 	    $self->{dbh}->do("CREATE TABLE amazon_key_exists (key TEXT PRIMARY KEY, value TEXT)");
 	};
-	die "Error: $@" if $@ && $@ !~ /table brackupcache already exists/;
+	die "Error: $@" if $@ && $@ !~ /table amazon_key_exists already exists/;
     }
 
     $self->{access_key_id} = $confsec->value("aws_access_key_id")   or die "No 'aws_access_key_id'";
@@ -51,13 +51,15 @@ sub has_chunk {
     if (my $dbh = $self->{dbh}) {
 	my $ans = $dbh->selectrow_array("SELECT COUNT(*) FROM amazon_key_exists WHERE key=?", undef, $dig);
 	warn "amazon database for $dig is = $ans\n";
-	return $ans;
+	return 1 if $ans;
     }
 
     my $res = eval { $self->{s3}->head_key({ bucket => $self->{chunk_bucket}, key => $dig }); };
+    return 0 unless $res;
     return 0 if $@ && $@ =~ /key not found/;
-
-    return $res->{content_type} eq "x-danga/brackup-chunk";
+    return 0 unless $res->{content_type} eq "x-danga/brackup-chunk";
+    $self->_cache_existence_of($dig);
+    return 1;
 }
 
 sub store_chunk {
@@ -66,38 +68,34 @@ sub store_chunk {
     my $blen = $chunk->backup_length;
     my $len = $chunk->length;
 
-    my $rv = $self->{s3}->add_key({
+    my $rv = eval { $self->{s3}->add_key({
 	bucket        => $self->{chunk_bucket},
 	key           => $dig,
 	value         => ${ $chunk->chunkref },
 	content_type  => 'x-danga/brackup-chunk',
-    });
-    warn "store chunk = $rv\n";
+    }) };
+    return 0 unless $rv;
+    $self->_cache_existence_of($dig);
+    return 1;
+}
 
-    use Data::Dumper;
-    warn Dumper($rv);
-
-    # until add_key's return value is good, let's just double-check it made it before we update our cache
-    if ($self->has_chunk($chunk)) {
-	if (my $dbh = $self->{dbh}) {
-	    $dbh->do("INSERT INTO amazon_key_exists VALUES (?,1)", undef, $dig);
-	}
-	return 1;
+sub _cache_existence_of {
+    my ($self, $dig) = @_;
+    if (my $dbh = $self->{dbh}) {
+        $dbh->do("INSERT INTO amazon_key_exists VALUES (?,1)", undef, $dig);
     }
-    return 0;
 }
 
 sub store_backup_meta {
     my ($self, $name, $file) = @_;
 
-    my $rv = $self->{s3}->add_key({
+    my $rv = eval { $self->{s3}->add_key({
 	bucket        => $self->{backup_bucket},
 	key           => $name,
 	value         => $file,
 	content_type  => 'x-danga/brackup-meta',
-    });
+    })};
 
-    warn "backup meta = $rv\n";
     return $rv;
 }
 
